## Copyright (C) 2000, 2001, 2002  SWsoft
## All rights reserved.
## Class for Payment plugins
package HSPC::MT::PluginConfig::PP;

use strict;

use SWsoft::protect;

use HSPC::DataRequest;
use HSPC::WebDB qw(select_rows atrans select_run);
use HSPC::Logger qw(sw_die sw_log);
use HSPC::MT::PM::Tools;
use HSPC::Localization;

use HSPC::PluginToolkit::General qw(throw_exception);

use HSPC::MT::PluginConfig;
use vars qw(@ISA);
@ISA = qw( HSPC::MT::PluginConfig );

use constant CLASS_INFO => {
    props => {
		'id'          => {
							col   => 'id',
							table => 'plugin',
							key   => 'AUTO',
							type  => 'int',
						},
		'sort_order'  => {
							col   => 'sort_order',
							table => 'plugin_pp',
							type  => 'int',
						},
		'banner_text' => {
							col        => 'banner_text',
							table      => 'plugin_pp',
							type       => 'char',
							max_length => 100000
						},
		'is_preauth_enabled'  => {
							col        => 'is_preauth_enabled',
							table      => 'plugin_pp',
							type       => 'int',
						},
		'sub_type'    => { 
							sql => 'UCASE(LEFT(plugin.template_id,INSTR(plugin.template_id, "_")-1))'
						},
		'paymethod_types'   => {
							type       => 'list', 
							class_info => 'PAYMETHOD_TYPE',
							join       => {'config_id'=>'id'}
						},
		'is_direct' => {
							col        => 'is_direct',
							table      => 'plugin_pp',
							type       => 'int',
						},
		'start_billing_trans' => {
							col        => 'start_billing_trans',
							table      => 'plugin_pp',
							type       => 'int',
						},
	},
	tables => {
		'plugin'    => { name => 'plugin', read_only => 0 },
		'plugin_pp' => {
			name       => 'plugin_pp',
			read_only  => 0,
			rel_table  => 'plugin',
			delete	   => 1,
			join_where => 'plugin.id = plugin_pp.config_id',
		},
	}
};

use constant PAYMETHOD_TYPE => {
    props=>{
		'config_id'   => {col => 'config_id'   , table => 'data' , key => 1},
		'type'        => {col => 'type'        , table => 'data' , type => 'char'},
		'template_id' => {col => 'template_id' , table => 'data' , type => 'char'},
    },
    tables => {
    	'data'  => {name => 'plugin_pp_paytypes'},
    }
};

sub save {
	my $self = shift;

	if (!defined ($self->sort_order())) {
		my $sql = q|
			SELECT
				MAX(plugin_pp.sort_order)
			FROM
				plugin, plugin_pp
			WHERE plugin.id = plugin_pp.config_id
				AND plugin.plugin_type = '| . $self->plugin_type() . q|'
				AND UCASE(LEFT(plugin.template_id,INSTR(plugin.template_id, "_")-1)) = '| . $self->sub_type() . q|'
		|;
		my $data = select_rows($sql);

		my $max_sort_order = $data && $data->[0] ? $data->[0]->[0] : 0;

		$self->sort_order(++$max_sort_order);
	}

	$self->is_direct(
		scalar @{$self->get_supported_payment_method_types()} ? 1:0
		) unless defined $self->is_direct();

	my $res = $self->SUPER::save();

	return $res;
}

sub get_supported_payment_method_types {
	my $self = shift;
	
	my $template = $self->find_mt_template;
	my @paytypes;

	if (UNIVERSAL::can($template, "get_supported_payment_method_types")) {
		my @paytype_opt;
		my $supported_ccs;

		eval {
			$supported_ccs = $template->get_supported_payment_method_types();
		};
		my $types = HSPC::MT::PM::Tools->get_pm_template_description(
			types => $supported_ccs,
		);
		@paytypes = map {[$_->{type}, string( $_->{type_title_id} ), $_->{image_id} ]} 
					@{$types};
	}
	return \@paytypes;
}


sub paymethod_types {
	my $self = shift;

	my $list = shift;

	if ($list) {
		my $types = HSPC::MT::PM::Tools->get_pm_template_description(
			types => $list,
		);
		return $self->SUPER::paymethod_types($types);
	} else {
		my $list = $self->SUPER::paymethod_types();
	
		my @types = map {$_->{type}} @{$list};

		return HSPC::MT::PM::Tools->get_pm_template_description(
						types => \@types
					);
	}
}

sub delete {
	my $self = shift;
	
	my $error = $self->SUPER::delete();
	if($error) {
		return $error;
	}

	my $sql = q|
		UPDATE
			plugin_pp, plugin
		SET
			sort_order = sort_order - 1
		WHERE
			plugin_pp.config_id = plugin.id and
			sort_order > | . $self->sort_order() . q| and
			vendor_id =  | . $self->vendor_id()  . q| and
			UCASE(LEFT(plugin.template_id,INSTR(plugin.template_id, "_")-1)) = '| . $self->sub_type() . q|'
	|;
	select_run($sql);
}

sub find_plugins {
	my $class = shift;
	my %h = (
		vendor_id        => undef,
		force_permission => undef,
		force_activity   => undef,
		plugin_type      => undef,
		sub_type         => undef,
		@_
	);

	my $force_activity   = $h{force_activity};
	my $force_permission = $h{force_permission};
	my $plugin_type      = $h{plugin_type};
	my $vendor_id        = $h{vendor_id};
	my $sub_type         = $h{sub_type};

	throw_exception(
		type        => 'error',
		log_message => 'vendor_id is a mandatory parameter'
	) unless $vendor_id;

	throw_exception(
		type        => 'error',
		log_message => 'plugin_type is a mandatory parameter'
	) unless $plugin_type;
		
	my $template_ids = $class->find_templates(
		plugin_type      => $plugin_type,
		sub_type         => $sub_type,
		vendor_id        => $vendor_id,
		force_permission => $force_permission,
	);

	my $request = HSPC::DataRequest->new(
		cols => [
			{ code => 'id' },
			{ code => 'vendor_id',   for_where => 1, value => $vendor_id },
			{ code => 'plugin_type', for_where => 1, value => $plugin_type },
			{ code => 'sort_order' }
		],
		select_as_hashes => 1,
		sort_col_code    => 'sort_order'
	);

	$request->set_col(code => 'is_active', for_where => '=', value => 1)
		unless $force_activity;

	$request->set_col(
		code => 'template_id', for_where => 1, value => $template_ids
	) if ( $template_ids && scalar(@{$template_ids}) );

	return $class->SUPER::find_plugins( request => $request, %h);
}

## Find templates installed in system
## for $plugin_type plugins
## Return array of hashes - template_id => module name
sub find_templates{
	my $class = shift;
	my %h = (
		plugin_type      => undef,
		vendor_id        => undef,
		force_permission => undef,
		sub_type         => undef,
		@_
	);

	my $sub_type = uc($h{sub_type});

	my $template_ids = $class->SUPER::find_templates(%h);

	my @templates = $sub_type ?
		grep { /^$sub_type\_/ } @$template_ids :
		@$template_ids;

	return \@templates;
}

1;
